/*
 * display.c
 *
 *  Created on: 21 avr. 2023
 *      Author: Flax
 */

// ==============
// Local includes
// ==============
#include "display.h"
#include "register.h"

// ===============
// Local constants
// ===============
#define cDISPMaxDigit		(3U)
#define cDISPValueMax		(999U)

#define cDISPSegSwitchDelay (2U)

// Division-less decade separation : https://www.baghli.com/hex2dec.php
#define cDISPDecadeSeparation100Num		((uint32_t)5243) // m
#define cDISPDecadeSeparation100Denom	((uint32_t)65536*8) // 2^s * 2^W
#define cDISPDecadeSeparation10Num		((uint32_t)52429) // m
#define cDISPDecadeSeparation10Denom	((uint32_t)65536*8) // 2^s * 2^W

const static uint8_t cDISPSevenSegmentTable_U8A[] =
{ // Segment order : 		B P A C F D G E
		0xBDU, // 0 ->  	1 X 1 1 1 1 0 1
		0x90U, // 1 ->		1 X 0 1 0 0 0 0
		0xA7U, // 2 ->		1 X 1 0 0 1 1 1
		0xB6U, // 3 -> 		1 X 1 1 0 1 1 0
		0x9AU, // 4 ->		1 X 0 1 1 0 1 0
		0x3EU, // 5 ->		0 X 1 1 1 1 1 0
		0x3FU, // 6 ->		0 X 1 1 1 1 1 1
		0xB0U, // 7 ->		1 X 1 1 0 0 0 0
		0xBFU, // 8 ->		1 X 1 1 1 1 1 1
		0xBEU, // 9 ->		1 X 1 1 1 1 1 0
		0x2FU, // E ->		0 X 1 0 1 1 1 1
		0x03U, // R ->		0 X 0 0 0 0 1 1
		0x3EU, // S ->		0 X 1 1 1 1 1 0
		0xABU, // P ->		1 X 1 0 1 0 1 1
		0x0FU, // T ->		0 X 0 0 1 1 1 1
		0xBDU, // O ->		1 X 1 1 1 1 0 1
		0x2BU, // F ->		0 X 1 0 1 0 1 1
		0x1FU, // B ->		0 X 0 1 1 1 1 1
		0x3DU, // G ->		0 X 1 1 1 1 0 1
		0xBBU, // A ->		1 X 1 1 1 0 1 1
		0x13U, // N ->		0 X 0 1 0 0 1 1
};

// ===============
// Local variables
// ===============
static uint8_t DISPSevenSegBuffer_U8A [cDISPMaxDigit];
static uint8_t DISPSevenSegSelect_U8;
static bool DISPEnableDisplay_B;
static uint16_t DISPSegSwitchDelayCnt_U16;

// ===========================
// Local functions prototypes
// ===========================
bool DISPValueToDigits (uint16_t value_u16, uint8_t* digits_U8A);

// ===========================
// Local functions definitions
// ===========================

bool DISPValueToDigits (uint16_t value_u16, uint8_t* digits_U8A)
{
	bool ret_B = false;
	uint16_t value_local_u16;

	// Division-less decade separation : https://www.baghli.com/hex2dec.php
	if (value_u16 <= cDISPValueMax)
	{
		ret_B = true;
		value_local_u16 = value_u16;

		// Check for 1st digit
		if (value_local_u16 > 99U)
		{
			// Divide by 100
			digits_U8A[0U] = (uint8_t)((cDISPDecadeSeparation100Num * (uint32_t)value_local_u16) / cDISPDecadeSeparation100Denom);
			value_local_u16 = value_local_u16 - ((uint16_t)(digits_U8A[0U]) * 100U); // Remove 1st digit
		}
		else
		{
			digits_U8A[0U] = 0U;
		}

		// Check for 2nd digit
		if (value_local_u16 > 9U)
		{
			// Divide by 10
			digits_U8A[1U] = (uint8_t)((cDISPDecadeSeparation10Num * (uint32_t)value_local_u16) / cDISPDecadeSeparation10Denom);
			value_local_u16 = value_local_u16 - ((uint16_t)(digits_U8A[1U]) * 10U); // Remove 2nd digit
		}
		else
		{
			digits_U8A[1U] = 0U;
		}

		// Check for 3nd digit
		digits_U8A[2U] = (uint8_t)value_local_u16;
	}

	return ret_B;
}

// ============================
// Shared functions definitions
// ============================
void DISPInit (void)
{
	DISPSevenSegSelect_U8 = 0U;
	DISPEnableDisplay_B = false;
}

void DISPStart (void)
{
	mREGSpi1InterruptRxEnable();
	mREGSpi1Enable();
	DISPEnableDisplay();
}

void DISPStop (void)
{
	mREGSpi1Disable();
	mREGSpi1InterruptRxDisable();
}

bool DISPSetDisplayDigit (uint8_t digit_u8, char digit_value_u8)
{
	bool ret_B = false;
	if (digit_u8 < (uint8_t)cDISPMaxDigit)
	{
		ret_B = true;
	}
	return (ret_B);
}

char DISPNumberToChar (uint8_t number_u8)
{
	char ret_C;

	return ret_C;
}



void DISPRefresh (void)
{
	// Switch active digit
	if (DISPSegSwitchDelayCnt_U16 < cDISPSegSwitchDelay)
	{
		DISPSegSwitchDelayCnt_U16++;
	}
	else
	{
		DISPSegSwitchDelayCnt_U16 = 0;
		if (DISPSevenSegSelect_U8 < cDISPMaxDigit)
		{
			DISPSevenSegSelect_U8++;
		}
		else
		{
			DISPSevenSegSelect_U8 = 0U;
		}
	}

	// Disable display before changing value
	mREGOutputsDisplayK1ResetState();
	mREGOutputsDisplayK2ResetState();
	mREGOutputsDisplayK3ResetState();
	mREGOutputsDisplayLATCHResetState();

	// Refresh
	mREGSpi1InterruptRxEnable();
	mREGSpi1SendFrame(DISPSevenSegBuffer_U8A[DISPSevenSegSelect_U8]);
}

void DISPEnableDisplay (void)
{
	DISPEnableDisplay_B = true;
}

void DISPDisableDisplay (void)
{
	DISPEnableDisplay_B = false;

	// Shut down all digits
	mREGOutputsDisplayK1ResetState();
	mREGOutputsDisplayK2ResetState();
	mREGOutputsDisplayK3ResetState();
}

void DISPInterruptRx (void)
{
	uint8_t rx_u8;
	if (DISPEnableDisplay_B == true)
	{ // Enable active digit
		switch (DISPSevenSegSelect_U8)
		{
		case 0U: // Enable K1
			mREGOutputsDisplayK1SetState();
			mREGOutputsDisplayK2ResetState();
			mREGOutputsDisplayK3ResetState();
			break;
		case 1U: // Enable K2
			mREGOutputsDisplayK1ResetState();
			mREGOutputsDisplayK2SetState();
			mREGOutputsDisplayK3ResetState();
			break;
		case 2U: // Enable K3
			mREGOutputsDisplayK1ResetState();
			mREGOutputsDisplayK2ResetState();
			mREGOutputsDisplayK3SetState();
			break;
		default: // Default state : disable display
			mREGOutputsDisplayK1ResetState();
			mREGOutputsDisplayK2ResetState();
			mREGOutputsDisplayK3ResetState();
			break;
		}

		mREGOutputsDisplayLATCHSetState(); // Latch value
	}
	else
	{ // Disable display
		mREGOutputsDisplayK1ResetState();
		mREGOutputsDisplayK2ResetState();
		mREGOutputsDisplayK3ResetState();
	}

	// Reset interrupt flag by reading the Rx Register
	rx_u8 = mREGSpi1ReadFrame();
}

bool DISPUpdateValueNumber (uint16_t value_U16)
{
	bool ret_B;
	uint8_t buff_U8A[cDISPMaxDigit];
	uint8_t cnt_U8;

	ret_B = DISPValueToDigits(value_U16, buff_U8A);

	for (cnt_U8 = 0U; cnt_U8 < cDISPMaxDigit; cnt_U8++)
	{
		DISPSevenSegBuffer_U8A[cnt_U8] = cDISPSevenSegmentTable_U8A[buff_U8A[cnt_U8]];
	}

	return ret_B;
}

void DISPUpdateDigit (uint8_t digit_U8, e_DISPChar7SegType char_E)
{
	if (digit_U8 < cDISPMaxDigit)
	{
		DISPSevenSegBuffer_U8A[digit_U8] = cDISPSevenSegmentTable_U8A[(uint8_t)char_E];
	}
}


